
var Const = {
  distance : { km : 1, mile : 0.621371192 },
  height   : { cm : 1, inch : 0.393700787 },
  weight   : { kg : 1, lb   : 2.20462262 },
  
  // wait for at least 2 consecutive GPS data before declaring the GPS ready
  gps_max_errors : 2
  

};

var user = (function () {
  var n = null;
  try {
    var n = Util.store.get("track_gps_user");
    if (n === null) {
      Util.log("Creating dummy user");
      n = {
      	name  : "DummyUser",
      	userId : "dummy@example.com",
      	passwd: "foobar",
        email : "dummy@example.com",
        weight : 73,
        height : 180,
        prefs : {
            weight   : "kg", 
            height   : "cm",
            distance : "km",
            workout  : "http://dbpedia.org/resource/Running"
        }
      };
    } else {
      Util.log("Found user " + n.userId);
    }
    n.getPostRunURI = function() { return "/user/" + this.userId + "/workout"; }
    n.getProfileURI = function() { return "/user/" + this.userId + "/profile"; }
 
    n.save = function() {
      if (Util.store.set("track_gps_user", this)) {
        Util.log("Saved user preferences");
      } else {
        Util.log("Failed to save preferences");
      }
      
    };
  } catch (e) { Util.log("[ERROR] user " + e); }
    return n; 
})();

function Activity(type) {
 this.activity  = type;
 this.date      = 0;
 this.distance  = 0;
 this.duration  = 0;
 this.userAgent = navigator.userAgent;
 this.events    = [];
}

var app = {

 // identifier for geolocation watch operation
 watchId : 0,

 // is an activity ongoing?
 inActivity : false,

 // total real time duration
 duration : 0,

 // start_time since start or pause
 start_time : 0,

 // identifier for setInterval timer operation
 intervalId : 0,

 // got_errors counts how many we've got
 got_error : Const.gps_max_errors,
 
 // total of geolocation data discarded
 discarded : 0,

 // total of geolocation data accepted
 locations : 0,

 // this will store all the geolocation data and pause
 currentActivity : new Activity(user.prefs.workout)
};

// add a geolocation position into the current activity
app.coordinates_event = function (activity, position) {
   var events = activity.events;
   events[events.length] =
    { t:position.timestamp, c: [position.coords.longitude, position.coords.latitude, position.coords.altitude] };
}

// add a pause event into the current activity
app.pause_event = function (activity) {
   var events = activity.events;
   events[events.length] = { t:Date.now(), s: [ "pause" ] };
}

// simply refresh the current screen data
app.repaint = function () {
  if (document.visibilityState && app.inActivity) {
    document.getElementById("locations").textContent = String(app.locations);
    var d = Util.geo.calculateDistanceCoordinates(app.currentActivity.events) * Const.distance[user.prefs.distance];
    var t = app.duration + (Date.now() - app.start_time);
    document.getElementById("distance").textContent = String(Math.round(d*100)/100);
    if (t != 0 && d != 0) {
      document.getElementById("speed").textContent = String(Math.round( (d / (t / 3600000)) * 100) / 100);
      document.getElementById("pace").textContent = String(Math.round( ((t / 60000)/d) * 100) / 100);
    }
    if (app.currentActivity.events.length > 0) {
      document.getElementById("gps_text").style.display = "none";
      document.getElementById("gps_data").style.display = "inline";
    }
  }
}

// Display a GPS location in the page
app.logGPS = function (position, startMsg) {
    Util.log(
      startMsg + position.timestamp 
       + ": " + position.coords.latitude
       + ", " + position.coords.longitude
       + ", " + position.coords.altitude
       + ", " + position.coords.accuracy
       + ", " + position.coords.altitudeAccuracy
       + ", " + position.coords.heading
       + ", " + position.coords.speed);
}


// when watchPosition is successful at returning a GPS coordinate
app.geoSuccess = function (position) {
  if (app.inActivity) {
   app.logGPS(position, "GPS: ");

    // discard positions with no altitude since they're likely inaccurate
    if (position.coords.altitude != null) {
      app.coordinates_event(app.currentActivity, position);
      app.locations++;
      app.repaint();
    } else {
      document.getElementById("discarded").textContent = String(app.discarded++);
    }
   } else {


     if (position.coords.altitude == null) {
       app.got_error++;
       if (app.got_error > Const.gps_max_errors) app.got_error = Const.gps_max_errors;
           app.logGPS(position, "[ERROR] GPS: ");
     } else {
       app.got_error--;
       app.logGPS(position, "GPS: ");
     }
     if (app.got_error <= 0) {
       app.got_error = 0;
       document.getElementById("gps_text").textContent = "ready";       
       document.getElementById("waves").style.display = "block";       
     } else {
       document.getElementById("gps_text").textContent = "not ready";
       document.getElementById("waves").style.display = "none";       
     }
   }
}

// when watchPosition is not successful at returning a GPS coordinate
app.geoError = function (error) {
  var code = "unknown";
  switch(error.code) {
    case error.TIMEOUT:
      code = "timeout";
      break;
    case error.PERMISSION_DENIED:
      code = "permission denied";
      break;
    case error.POSITION_UNAVAILABLE:
      code = "position unavailable";
      break;
  }
  if (app.inActivity) {
    document.getElementById("errors").textContent = String(++app.got_error);
    Util.log("[ERROR] GPS: " + code); 
  } else {
    document.getElementById("gps_text").textContent = code; 
    // wait for MAX_ERRORS non-consecutive errors
    app.got_error = Const.gps_max_errors;
  }
}

// Called by setInterval to display the real time
app.timer = function () {
  if (document.visibilityState) {
    document.getElementById("realtime").textContent = 
      Util.formatTime(Math.round((app.duration+(Date.now()-app.start_time))/1000));
  }
}

// The Start / Pause button
app.start = function () {
  if (!app.inActivity) {
    // start an activity
   app.inActivity = true;
   var now = Date.now();
   app.start_time = now;
   app.intervalId = setInterval(app.timer, 1000);
   document.getElementById("stop").disabled = false;
   document.getElementById("start").textContent = "Pause";
   if (app.currentActivity.time === 0) {
     app.currentActivity.date = now;
   } 
  } else {
   // pause an activity
   app.inActivity = false;
   clearInterval(app.intervalId);
   app.duration += (Date.now() - app.start_time);
   document.getElementById("start").textContent = "Restart";
   app.pause_event(app.currentActivity);
   document.getElementById("gps_text").style.display = "inline";
   document.getElementById("gps_data").style.display = "none";
  }
}

// The Stop button
app.stop = function () {
  app.inActivity = false;
  navigator.geolocation.clearWatch(app.watchId);
  clearInterval(app.intervalId);
  app.currentActivity.duration += (Date.now() - app.currentActivity.time);
  app.currentActivity.distance = (Util.geo.calculateDistanceCoordinates(app.currentActivity.events) * 1000).toFixed(0);
  app.got_error = 0;
  app.locations = 0;
  app.discarded = 0;
  app.duration = 0;
  document.getElementById("start").textContent = "Start";
  document.getElementById("stop").disabled = true;
  if (app.currentActivity.events.length > 2) {
  	var success = function (xhr) {
  		if (xhr.status == 201) {
  			Util.log("Resource created at " + xhr.getResponseHeader("Location"));
  		}
  	}
  	var error = function(obj) {
  		if (!Util.store.set("track_gps_app", app.currentActivity)) {
      		Util.log("[ERROR] Can't store the workout");
    	}
  	}
  	Util.postAsJSON(user.getPostRunURI(),
  	                { userId : user.userId, passwd : user.passwd, data : app.currentActivity },
  	                success, error);
  } else {
  	Util.log("[ERROR] Your workout doesn't contain enough events");
  }
  app.currentActivity = new Activity(user.prefs.workout);
}

app.init = function () { 
   
  if (typeof(document.visibilityState) !== "undefined")
    document.addEventListener("visibilitychange", app.repaint, false);
  else
    document.visibilityState = true;
  
  // @@ in the future, make sure this is empty
  if (!Util.store.enabled) {
    Util.log("[ERROR] no local storage");
  }
  app.setPrefs();

  if (!!navigator.geolocation) {
    app.watchId = navigator.geolocation.watchPosition(app.geoSuccess, app.geoError, {enableHighAccuracy:true, maximumAge:0, timeout:1000});
  } else {
    Util.log("[ERROR] no geolocation support");
    document.getElementById("gps_text").textContent = "not supported";
    document.getElementById("start").disabled = true;
  }

}

window.addEventListener("load", app.init, false);

app.changeDistanceUnit = function () {
  var node = firstElementChild(document.getElementById("select_distance"));
  do {
    if (node.selected) {
       user.prefs.distance = node.getAttribute("value");
    }
  } while ((node=nextElementSibling(node)) != null);
  app.setPrefs();
  app.repaint();
}

app.changeWeightUnit = function () {
  var node = firstElementChild(document.getElementById("select_weight"));
  do {
    if (node.selected) {
       user.prefs.weight = node.getAttribute("value");
    }
  } while ((node=nextElementSibling(node)) != null);
  app.setPrefs();
  app.repaint();
}

app.changeHeightUnit = function () {
  var node = firstElementChild(document.getElementById("select_height"));
  do {
    if (node.selected) {
       user.prefs.height = node.getAttribute("value");
    }
  } while ((node=nextElementSibling(node)) != null);
  app.setPrefs();
  app.repaint();
}

app.changeWorkout = function () {
  var node = firstElementChild(document.getElementById("select_workout"));
  do {
    if (node.selected) {
       user.prefs.workout = node.getAttribute("value");
       app.currentActivity = new Activity(user.prefs.workout);
    }
  } while ((node=nextElementSibling(node)) != null);
}

app.setPrefs = function () {
  var nodes = null, node = null;

  document.getElementById("user_name_input").value = user.name;
  document.getElementById("user_email_input").value = user.email;

  nodes = document.getElementsByClassName("distance_unit");
  i = 0;
  do {
    nodes[i].textContent = user.prefs.distance;
  } while (++i < nodes.length);
  node = firstElementChild(document.getElementById("select_distance"));
  do {
    if (node.getAttribute("value")===user.prefs.distance) node.selected = true;
  } while ((node=nextElementSibling(node)) != null);
  
  nodes = document.getElementsByClassName("weight_unit");
  i = 0;
  do {
    nodes[i].textContent = user.prefs.weight;
  } while (++i < nodes.length);
  node = firstElementChild(document.getElementById("select_weight"));
  do {
    if (node.getAttribute("value")===user.prefs.weight) node.selected = true;
  } while ((node=nextElementSibling(node)) != null);
  document.getElementById("weight_input").value = user.weight;

  nodes = document.getElementsByClassName("height_unit");
  i = 0;
  do {
    nodes[i].textContent = user.prefs.height;
  } while (++i < nodes.length);
  node = firstElementChild(document.getElementById("select_height"));
  do {
    if (node.getAttribute("value")===user.prefs.height) node.selected = true;
  } while ((node=nextElementSibling(node)) != null);
  document.getElementById("height_input").value = user.height;

  node = firstElementChild(document.getElementById("select_workout"));
  do {
    if (node.getAttribute("value")===user.prefs.workout) node.selected = true;
  } while ((node=nextElementSibling(node)) != null);
}

app.save = function () {
  user.name = document.getElementById("user_name_input").value;
  user.email = document.getElementById("user_email_input").value;
  user.weight = parseInt(document.getElementById("weight_input").value);
  user.height = parseInt(document.getElementById("height_input").value);
  user.save();
}
