package org.w3.virtualtrainer.model

import com.novus.salat._
import com.novus.salat.global._
import com.mongodb.casbah.Imports._
import com.novus.salat.annotations._

import com.novus.salat.dao._
import com.mongodb.casbah.MongoConnection
  
/*
{
 activity:"http://dbpedia.org/resource/Running",
 events:
 [
  {
   t: 1314049555597,
   c: [-71.09068298339844,42.361671924591064,66]
  },
  {
   t: 1314049559622,
   s: "pause"
  },
  {
   t: 1314549555597,
   c: [-71.09068298339844,42.361671924591064,66]
  },
   t: 1315049559622,
   s: "pause"
  }
 ]
}
*/

case class Workout(@Key("_id") id: ObjectId = new ObjectId,
                   user:String,
                   activity:String,
                   date: Long,
                   distance: Long,
                   duration: Long,
                   @Key("userAgent") useragent: String,
                   events:Seq[Event]) {

  def asJSON:String = {
    val formattedEvents = events map { _.asJSON } mkString ("[ ", ", ", " ]")
    """{ "activity": "%s", "date": %d, "distance": %d, "duration": %s, "userAgent": "%s", "events": %s }""" format
         (activity,         date,       distance,       duration,       useragent,         formattedEvents)
  }

}

case class Event(@Key("t") timestamp:Long,
                 @Key("c") coordinates:Option[(Double, Double, Option[Double])],
                 @Key("s") status:Option[String]) {
 
  def c:Option[(Double, Double, Option[Double])] = {
    coordinates.asInstanceOf[Option[com.mongodb.BasicDBList]] map { dblist => {
      (dblist:MongoDBList).toList match {
        case lon :: lat :: alt :: Nil => (lon.toString.toDouble, lat.toString.toDouble, Some(alt.toString.toDouble))
        case lon :: lat :: Nil => (lon.toString.toDouble, lat.toString.toDouble, None)
        case _ => sys.error("here")
      }
    } }
  }
  
  def asJSON:String = {
    val tString = """"t": "%s"""" format timestamp.toString
    val cString = {
      c map {
        case (lon, lat, Some(alt)) => """, "c": [%f, %f, %f]""" format (lon, lat, alt)
        case (lon, lat, None) => """, "c": [%f, %f]""" format (lon, lat)
      } getOrElse ""
    }
    val sString = status map { s =>  """, "s": "%s"""" format s.toString } getOrElse ""
    "{ %s%s%s }" format (tString, cString, sString)
  }
  
}


object WorkoutDAO extends SalatDAO[Workout, ObjectId](collection = MongoConnection("gw42.w3.org", 27017)("virtual_trainer")("user.workouts")) {
  
  def getAllWorkoutsByUser(user:String):Seq[Workout] = {
    val results = find(ref = MongoDBObject("user" -> user)).toSeq
    results
  }
  
  def findOneByID(id:String):Option[Workout] = {
    try {
      val objectId = new ObjectId(id)
      findOneByID(objectId)
    } catch {
      case iae:IllegalArgumentException => None
    }
    
  }
  
}
